/* packet-adh.c
 * Routines for Ananas adhoc dissection 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <glib.h>

#include <epan/packet.h>
#include <epan/dissectors/packet-ip.h>

#include <etypes.h>

#define ADHOC_ADDRESS_SIZE_IN_BYTES  	8
#define ADHOC_ADDRESS_STRING_SIZE	20

static int proto_adh = -1;
static int hf_adh_dst = -1;
static int hf_adh_src = -1;
static int hf_adh_hop_count = -1;
static int hf_adh_prot_id = -1;
static int hf_adh_seq = -1;
static int hf_adh_flags = -1;
static int hf_adh_wrap = -1;
static dissector_handle_t up_proto_handle;

/*subtree pointers*/
static gint ett_adh = -1;

/*write a byte adhoc address to a string (repr : AAAA:BBBB:CCCC:DDDD)*/
int wrt_adhbytes_to_str(guint8 * baddr, char * straddr)
{
	sprintf(straddr
		,"%.2x%.2x:%.2x%.2x:%.2x%.2x:%.2x%.2x"
		,baddr[0]
		,baddr[1]
		,baddr[2]
		,baddr[3]
		,baddr[4]
		,baddr[5]
		,baddr[6]
		,baddr[7]);
		
	return 1;
}

/*dissect the packets*/
static void dissect_adh(tvbuff_t * tvb, packet_info *pinfo, proto_tree * tree)
{
	proto_item * ti;
	proto_tree * adh_tree = tree;
	proto_tree * adh_son_tree;
	guint8 dst_address [ADHOC_ADDRESS_SIZE_IN_BYTES];
	guint8 src_address [ADHOC_ADDRESS_SIZE_IN_BYTES];
	char dst_text [ADHOC_ADDRESS_STRING_SIZE];
	char src_text [ADHOC_ADDRESS_STRING_SIZE];
	guint8 i;
	guint16 offset = 0, offset_proto;
	guint16 hopcount;
	guint16 prot_id;
	guint16 seq;
	guint8  flags;
	guint8  wrap;
	/* tvbuff_t * upper_tvb;*/ /*buffer sent to the uppper protocol*/
	
	if (check_col(pinfo->cinfo, COL_PROTOCOL))
		col_set_str(pinfo->cinfo, COL_PROTOCOL, "ADH");

	/*add an item to the tree to add a subtree*/
	ti = proto_tree_add_item(adh_tree, 0, tvb, 0, 24, 0);
    	/*add the subtree with the adhoc protocol infos*/
	adh_son_tree = proto_item_add_subtree(ti, proto_adh);
	
	/*get dest and src adhoc addresses from the frame */
	for (i = 0; i < ADHOC_ADDRESS_SIZE_IN_BYTES; i++)
	{
		dst_address[i] = tvb_get_guint8(tvb,i);
		src_address[i] = tvb_get_guint8(tvb,i+ADHOC_ADDRESS_SIZE_IN_BYTES);
	}
	/*convert bytes addresses to texts*/
	wrt_adhbytes_to_str(dst_address,dst_text);
	wrt_adhbytes_to_str(src_address,src_text);

	//set the summary information	
	proto_item_append_text(ti,"Adh Protocol, Src Addr: %s, Dest Addr: %s",src_text, dst_text);
	
	/*add source and dest adhoc addresses to the subtree */
	proto_tree_add_text(adh_son_tree, tvb, offset,ADHOC_ADDRESS_SIZE_IN_BYTES,"Dest Adhoc Addr: %s",dst_text);
	offset += ADHOC_ADDRESS_SIZE_IN_BYTES;
	
	proto_tree_add_text(adh_son_tree, tvb, offset,ADHOC_ADDRESS_SIZE_IN_BYTES,"Src Adhoc Addr: %s",src_text);
	offset += ADHOC_ADDRESS_SIZE_IN_BYTES;
	
	/* get the protocol ID and the hop count from the frame and add it to the subtree*/
	prot_id = tvb_get_ntohs(tvb,offset);
	offset_proto = offset;
	/*proto_tree_add_text(adh_son_tree, tvb, offset, sizeof(guint16), "Protocol Id: 0x%.4x (%d)",prot_id,prot_id);*/
	offset += sizeof(guint16);
	
	hopcount = tvb_get_ntohs(tvb,offset);
	proto_tree_add_text(adh_son_tree, tvb, offset, sizeof(guint16), "Hop Count: 0x%.4x (%d)",hopcount,hopcount);
	offset += sizeof(guint16);
	
	flags = tvb_get_guint8(tvb, offset);
	proto_tree_add_text(adh_son_tree, tvb, offset, sizeof(guint8), "Flags : 0x%.2x", flags);
	offset += sizeof(guint8);
	
	wrap = tvb_get_guint8(tvb, offset);
	proto_tree_add_text(adh_son_tree, tvb, offset, sizeof(guint8), "Wrap : 0x%.2x (%d)", wrap, wrap);
	offset += sizeof(guint8);
	
	/*get the Groupe/Sequence and add it to the subtree*/
	seq = tvb_get_ntohs(tvb,offset);
	proto_tree_add_text(adh_son_tree, tvb, offset, sizeof(guint16), "Sequence: 0x%.4x (%d)", seq, seq);
	offset += sizeof(guint16);

	ethertype(prot_id, tvb, offset, pinfo, adh_tree, adh_son_tree, hf_adh_prot_id, -1, 0);
	
	
}

void proto_register_adh(void)
{
	static hf_register_info hf[] = 
	{
		{ &hf_adh_dst,
			{ "Destination"
			 ,"adh.dst"
			 ,FT_STRING
			 ,BASE_NONE
			 ,NULL
			 ,0x0
			 ,"Destination adhoc address"
			 ,HFILL	
			} 
		},
		{ &hf_adh_src,
			{ "Source"
			 ,"adh.src"
			 ,FT_STRING
			 ,BASE_NONE
			 ,NULL
			 ,0x0
			 ,"Source adhoc address"
			 ,HFILL	

			} 
		},/*
		{ &hf_adh_prot_id,
    			{ "Protocol"
			 ,"adh.protocol"
			 ,FT_STRING
			 ,BASE_NONE
			 ,NULL
			 ,0x0
			 ,"Protocol Id"
			 ,HFILL 
			 }
		},*/
		{ &hf_adh_prot_id,
			{ "Protocol"
			 ,"adh.type"
			 ,FT_UINT16
			 ,BASE_HEX
			 ,VALS(etype_vals)
			 ,0x0
			 ,"Protocol Id"
			 ,HFILL
			 }
		},
		{ &hf_adh_hop_count,
			{ "HopCount"
			 ,"adh.hopcount"
			 ,FT_STRING
			 ,BASE_NONE
			 ,NULL
			 ,0x0
			 ,"Hop Count"
			 ,HFILL	
			} 
		},
		{ &hf_adh_flags,
			{ "Flags"
			 ,"adh.flags"
			 ,FT_STRING
			 ,BASE_NONE
			 ,NULL
			 ,0x0
			 ,"Group sequence"
			 ,HFILL	
			} 
		},
		{ &hf_adh_wrap,
			{ "Wrap"
			 ,"adh.wrap"
			 ,FT_STRING
			 ,BASE_NONE
			 ,NULL
			 ,0x0
			 ,"Wrap"
			 ,HFILL	
			} 
		},
		{ &hf_adh_seq,
			{ "Sequence"
			 ,"adh.seq"
			 ,FT_STRING
			 ,BASE_NONE
			 ,NULL
			 ,0x0
			 ,"Sequence"
			 ,HFILL	
			} 
		},
	};
	
	static gint * ett[] = {&ett_adh,};

	proto_adh = proto_register_protocol("Ananas Adhoc Protocol","adh","adh");
	
	proto_register_field_array(proto_adh, hf, array_length(hf));
	proto_register_subtree_array(ett,array_length(ett));
}

void proto_reg_handoff_adh(void)
{
	dissector_handle_t adh_handle;

	adh_handle = create_dissector_handle(dissect_adh, proto_adh);
	dissector_add("ethertype",0x0666,adh_handle);
	dissector_add("ethertype",0x0777,adh_handle);
}
