/* packet-adhctl.c
 * Routines for Ananas adhoc dissection 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <glib.h>

#include <epan/packet.h>
#include <epan/dissectors/packet-ip.h>

#include <etypes.h>

#define ADHOC_ADDRESS_SIZE_IN_BYTES  	8
#define ADHOC_ADDRESS_STRING_SIZE	20

#define ROUTE_REQUEST	1
#define ROUTE_RESPONSE	2

static const value_string command_vals[] = {
  { ROUTE_REQUEST,  "Route request" },
  { ROUTE_RESPONSE, "Route response" },
};


static int proto_adhctl = -1;
static int hf_adhctl_command = -1;
static int hf_adhctl_size = -1;
static int hf_adhctl_response_router = -1;
static dissector_handle_t up_proto_handle;

/*subtree pointers*/
static gint ett_adh = -1;

/*write a byte adhoc address to a string (repr : AAAA:BBBB:CCCC:DDDD)*/
int wrt_adhbytes_to_str_ctl(guint8 * baddr, char * straddr)
{
  sprintf(straddr
	  ,"%.2x%.2x:%.2x%.2x:%.2x%.2x:%.2x%.2x"
	  ,baddr[0]
	  ,baddr[1]
	  ,baddr[2]
	  ,baddr[3]
	  ,baddr[4]
	  ,baddr[5]
	  ,baddr[6]
	  ,baddr[7]);
		
  return 1;
}

/*dissect the packets*/
static void dissect_adhctl(tvbuff_t * tvb, packet_info *pinfo, proto_tree * tree)
{
  proto_item * ti;
  proto_tree * adh_tree = tree;
  proto_tree * adh_son_tree;
  guint8 router_address [ADHOC_ADDRESS_SIZE_IN_BYTES];
  //guint8 src_address [ADHOC_ADDRESS_SIZE_IN_BYTES];
  char router_text [ADHOC_ADDRESS_STRING_SIZE];
  //char src_text [ADHOC_ADDRESS_STRING_SIZE];
  guint8 i;
  guint16 offset = 0;//, offset_proto;
  //guint16 hopcount;
  //guint16 prot_id;
  //guint16 seq;
  //guint8  flags;
  //guint8  wrap;
  guint16 command, size;
  /*tvbuff_t * upper_tvb;*/ /*buffer sent to the upper protocol*/
  int nb_routers, ind_router;
	
  if (check_col(pinfo->cinfo, COL_PROTOCOL))
    col_set_str(pinfo->cinfo, COL_PROTOCOL, "ADH CTRL");

  /*add an item to the tree to add a subtree*/
  ti = proto_tree_add_item(adh_tree, 0, tvb, 0, tvb_length(tvb), 0);
  /*add the subtree with the adhoc protocol infos*/
  adh_son_tree = proto_item_add_subtree(ti, proto_adhctl);
	
  command = tvb_get_ntohs(tvb, offset);
  offset += sizeof(command);

  size = tvb_get_ntohs(tvb, offset);
  offset += sizeof(size);

  //add the summary : command and data length
  proto_item_append_text(ti, "Adh control protocol, command : 0x%x, size : %d", command, size);

  proto_tree_add_uint(adh_son_tree, hf_adhctl_command, tvb, offset - sizeof(size) - sizeof(command), sizeof(command), command);
  proto_tree_add_uint(adh_son_tree, hf_adhctl_size   , tvb, offset - sizeof(size)                  , sizeof(size)   , size   );

  switch (command) {
  case ROUTE_RESPONSE :
    nb_routers = size / ADHOC_ADDRESS_SIZE_IN_BYTES;
    for (ind_router = 0; ind_router < nb_routers; ind_router ++) {
      for (i = 0; i < ADHOC_ADDRESS_SIZE_IN_BYTES; i++)
	router_address[i] = tvb_get_guint8(tvb,offset+i);

      wrt_adhbytes_to_str_ctl(router_address,router_text);
      proto_tree_add_text(adh_son_tree, tvb, offset, ADHOC_ADDRESS_SIZE_IN_BYTES,"  Router %d : %s", ind_router + 1, router_text);
      offset += ADHOC_ADDRESS_SIZE_IN_BYTES;
    }
    break;
  }

  return;	
}

void proto_register_adhctl(void)
{
  static hf_register_info hf[] = 
    {
      { &hf_adhctl_command,
	{ "Adh control command"
	  ,"adhctl.command"
	  ,FT_UINT16
	  ,BASE_HEX
	  ,VALS(command_vals)
	  ,0x0
	  ,"Adh control command"
	  ,HFILL
	}
      },
      { &hf_adhctl_size,
	{ "Adh control size"
	  ,"adhctl.size"
	  ,FT_UINT16
	  ,BASE_DEC
	  ,NULL
	  ,0x0
	  ,"Adh control size"
	  ,HFILL
	}
      },
      { &hf_adhctl_response_router,
	{
	  "Route response router"
	  ,"adhctl.response_router"
	  ,FT_STRING
	  ,BASE_NONE
	  ,NULL
	  ,0x0
	  ,"Route response router"
	  ,HFILL
	}
      },

    };
	
  static gint * ett[] = {&ett_adh,};

  proto_adhctl = proto_register_protocol("Ananas Control Protocol","adhctl","adhctl");
	
  proto_register_field_array(proto_adhctl, hf, array_length(hf));
  proto_register_subtree_array(ett,array_length(ett));
}

void proto_reg_handoff_adhctl(void)
{
  dissector_handle_t adhctl_handle;

  adhctl_handle = create_dissector_handle(dissect_adhctl, proto_adhctl);
  dissector_add("ethertype",0x0667,adhctl_handle);
}
