/* packet-adholsr.c
 * Routines for Ananas adhoc dissection 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <glib.h>

#include <epan/packet.h>
#include <epan/dissectors/packet-ip.h>

#include <etypes.h>

#define ADHOC_ADDRESS_SIZE_IN_BYTES  	8
#define ADHOC_ADDRESS_STRING_SIZE	20

#define ADHOLSR_HEADER                  24

/* Initialize the protocol and registered fields */
static int proto_adholsr = -1;
static int hf_olsr_packet_len = -1;
static int hf_olsr_packet_seq_num = -1;
static int hf_olsr_message_type = -1;
static int hf_olsr_vtime = -1;
static int hf_olsr_message_size = -1;
static int hf_olsr_ttl = -1;
static int hf_olsr_hop_count = -1;
static int hf_olsr_message_seq_num = -1;

static int hf_olsr_htime = -1;
static int hf_olsr_willingness = -1;

static int hf_olsr_link_type = -1;
static int hf_olsr_link_message_size = -1;
static int hf_olsr_ansn = -1;

static int hf_olsr_origin_addr = -1;
static int hf_olsr_origin_addr_adh = -1;
static int hf_olsr_neighbor_addr = -1;
static int hf_olsr_neighbor_addr_adh = -1;
static int hf_olsr_interface_addr = -1;
static int hf_olsr_netmask	= -1;
static int hf_olsr_network_addr = -1;
static int hf_olsr_origin6_addr = -1;
static int hf_olsr_neighbor6_addr = -1;
static int hf_olsr_interface6_addr = -1;
static int hf_olsr_netmask6 = -1;
static int hf_olsr_network6_addr = -1;
static int hf_olsr_testlq_reserved = -1;
static int hf_olsr_testlq_seqnum = -1;
static int hf_olsr_tclq_lq  = -1;
static int hf_olsr_tclq_nlq = -1;

/* Initialize the subtree pointers*/
static gint ett_olsr = -1;
static dissector_handle_t up_proto_handle;

/*subtree pointers*/
static gint ett_adholsr = -1;

#define HELLO	1
#define TC	2
#define LQ	3
#define HNA	4
#define TC_LQ   5
#define UNKNOWN	6
#define TEST_LQ 7

static const value_string message_type_vals[] = {
  { HELLO, "HELLO" },
  { TC,    "TC" },
  { LQ,   "LQ" },
  { HNA,   "HNA" },
  { TC_LQ,   "TC_LQ" },
  { UNKNOWN, "UNKNOWN" },
  { TEST_LQ, "TEST_LQ" },
  { 0,     NULL }
};

/*write a byte adhoc address to a string (repr : AAAA:BBBB:CCCC:DDDD)*/
int wrt_adhbytes_to_str_adholsr(guint8 * baddr, char * straddr)
{
  sprintf(straddr
	  ,"%.2x%.2x:%.2x%.2x:%.2x%.2x:%.2x%.2x"
	  ,baddr[0]
	  ,baddr[1]
	  ,baddr[2]
	  ,baddr[3]
	  ,baddr[4]
	  ,baddr[5]
	  ,baddr[6]
	  ,baddr[7]);
		
  return 1;
}

static int
dissect_adholsr(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
{
  proto_item *ti;
  proto_tree *olsr_tree;
        
  int link_message_size, message_size, message_type, packet_size, seq_num;
  unsigned int position;
  int high_bits, low_bits, vtime, htime;
  double Vtime, Htime;
	
  guint16 packet_len;
	
  guint32 origin_addr;
  char origin_addr_adh[256];
  guint8 orig[8];
  guint32 neighbor_addr;
  //guint32 interface_addr;
  guint32 network_addr;
  guint32 netmask;
  int i;
  //int ind;
	
  if (check_col(pinfo->cinfo, COL_PROTOCOL))
    col_set_str(pinfo->cinfo, COL_PROTOCOL, "ADH_OLSR");

  /*add an item to the tree to add a subtree*/
  //ti = proto_tree_add_item(tree, 0, tvb, 0, 24, 0);
  ti = proto_tree_add_item(tree, 0, tvb, 0, tvb_length(tvb), 0);
  /*add the subtree with the adhoc protocol infos*/
  olsr_tree = proto_item_add_subtree(ti, proto_adholsr);
	
  proto_item_append_text(ti,"Adh OLSR Protocol");	

  position = 0;

  packet_len = tvb_get_ntohs(tvb, position);
  proto_tree_add_uint(olsr_tree, hf_olsr_packet_len, tvb, position, 2, packet_len);

  position += 2;
  seq_num = tvb_get_ntohs(tvb, position);
  proto_tree_add_uint(olsr_tree, hf_olsr_packet_seq_num, tvb, position, 2, seq_num);

  position += 2;

  packet_len = tvb->length - 24;

  packet_size = (packet_len -4) / 4;

  while (position < tvb_length(tvb)) {
    message_type = tvb_get_guint8(tvb, position);
    proto_tree_add_uint(olsr_tree, hf_olsr_message_type, tvb, position, 1, message_type);
			
    /*-------------Dissect Validity Time-------------------------*/
    vtime = tvb_get_guint8(tvb, position+1);
    high_bits = ((vtime & 0xF0) >> 4);
    low_bits = (vtime & 0x0F);
    Vtime = ((1<<low_bits)/16.0)*(1+(high_bits/16.0));
    proto_tree_add_double_format(olsr_tree, hf_olsr_vtime, tvb, position+1, 1, Vtime, "Validity Time: %.3f (in seconds)", Vtime);
			
    /*-------------Dissect Message Size---------------------------*/
    message_size = tvb_get_ntohs(tvb, position+2);
    if (message_size <= 0) {
      proto_tree_add_uint_format(olsr_tree, hf_olsr_message_size, tvb, position+2, 2, message_size,"Message Size: %u bytes (too short, must be >= 4)", message_size);
      break;
    }
    proto_tree_add_uint_format(olsr_tree, hf_olsr_message_size, tvb, position+2, 2, message_size,"Message Size: %u bytes", message_size);
			
    position += 4;
    message_size -= 4;

    /*-----------------Dissecting: Origin Addr, TTL, Hop Count, and Message Seq Number*/
    if(message_size > 0)	{
      tvb_memcpy(tvb, (guint8*)&origin_addr, position, 4);
      proto_tree_add_ipv4(olsr_tree, hf_olsr_origin_addr, tvb, position, 4, origin_addr);
      //message_size--;
      //packet_size--;
      position += 4;
      message_size -= 4;
    }

    //dissecting ADH originator address
    if (message_size > 0) {
      for (i = 0; i < 8; i++)
	orig[i] = tvb_get_guint8(tvb,i+position);
					
      wrt_adhbytes_to_str_adholsr(orig, origin_addr_adh);
	
      proto_tree_add_text(olsr_tree, tvb, position,8 ,"Originator Adhoc Addr: %s",origin_addr_adh);
      //message_size--;
      //packet_size--;
      message_size -= 8;
      position+=8;
    }

    if(message_size > 0)	{
      proto_tree_add_item(olsr_tree, hf_olsr_ttl, tvb, position, 1, FALSE);
      proto_tree_add_item(olsr_tree, hf_olsr_hop_count, tvb, position+1, 1, FALSE);
      proto_tree_add_item(olsr_tree, hf_olsr_message_seq_num, tvb, position+2, 2, FALSE);
      message_size -= 4;
      //packet_size--;
      position += 4;
    } 

    if (message_type == TEST_LQ) {
      proto_tree_add_uint(olsr_tree, hf_olsr_testlq_reserved, tvb, position, 2, tvb_get_ntohs(tvb, position));
      position += 2;
      proto_tree_add_uint(olsr_tree, hf_olsr_testlq_seqnum, tvb, position, 2, tvb_get_ntohs(tvb, position));
      position += 2;
      message_size -= 4;
    }
			
    /* --------------Dissecting TC message--------------------- */
    if(message_size>0 && message_type == TC)   {
      proto_tree_add_item(olsr_tree, hf_olsr_ansn, tvb, position, 2, FALSE);

      position+=4;
      message_size-=4;
      packet_size--;

      while (message_size > 0) {
	for (i = 0; i < 8; i++)
	  orig[i] = tvb_get_guint8(tvb,i+position);
					
	wrt_adhbytes_to_str_adholsr(orig, origin_addr_adh);
	
	proto_tree_add_text(olsr_tree, tvb, position,8 ,"Neighbor Adhoc Addr: %s",origin_addr_adh);
				  
	position += 8;
	message_size -= 8;
				  
	tvb_memcpy(tvb, (guint8*)&neighbor_addr, position, 4);
	proto_tree_add_ipv4(olsr_tree, hf_olsr_neighbor_addr, tvb, position, 4, neighbor_addr);
				  
	position += 4;
	message_size-=4;
				  
	packet_size--;
      }
    }

    /* --------------Dissecting TC LQ message--------------------- */
    if(message_size>0 && message_type == TC_LQ)   {
      int nb_neighbors;
      int loop;
      int before_tc;

      proto_tree_add_item(olsr_tree, hf_olsr_ansn, tvb, position, 2, FALSE);

      position += 4;
      message_size -= 4;
      packet_size -= 4;

      nb_neighbors = message_size / (4 + 8 + 4); //IP + ADH + LQ
      before_tc = position;
      for (loop = 0; loop < nb_neighbors; loop++) {
	guint8 lq, nlq;
	for (i = 0; i < 8; i++)
	  orig[i] = tvb_get_guint8(tvb,i+position);
					
	wrt_adhbytes_to_str_adholsr(orig, origin_addr_adh);
	
	proto_tree_add_text(olsr_tree, tvb, position, 8 ,"Neighbor Adhoc Addr: %s",origin_addr_adh);
				  
	position += 8;
	message_size -= 8;
				  
	tvb_memcpy(tvb, (guint8*)&neighbor_addr, position, 4);
	proto_tree_add_ipv4(olsr_tree, hf_olsr_neighbor_addr, tvb, position, 4, neighbor_addr);

	position +=4;
	message_size -= 4;

	lq  = tvb_get_guint8(tvb, before_tc + nb_neighbors * 12 + loop * 2);

	nlq = tvb_get_guint8(tvb, before_tc + nb_neighbors * 12 + nb_neighbors * 2 + loop * 2);

	proto_tree_add_uint(olsr_tree, hf_olsr_tclq_lq , tvb, before_tc + nb_neighbors * 12 + loop * 2    , 1, lq );
	proto_tree_add_uint(olsr_tree, hf_olsr_tclq_nlq, tvb, before_tc + nb_neighbors * 12 + nb_neighbors * 2 + loop * 2, 1, nlq);
				  
	packet_size--;
      }
      position += nb_neighbors * 4;
    }
    /* -------------Dissect LQ message----------------------- */
    else if(message_size>0 && message_type == LQ)  {
      /*---------------------Dissect LQ Emission Invertal-------------------*/
      htime = tvb_get_guint8(tvb, position + 2);
      high_bits = ((htime & 0xF0) >> 4);
      low_bits = (htime & 0x0F);
      Htime = ((1<<low_bits)/16.0)*(1+(high_bits/16.0));
      proto_tree_add_double_format(olsr_tree, hf_olsr_htime, tvb, position + 2, 1, Htime, "LQ Emission Interval: %.3f (in seconds)", Htime);
			   
				
      /*-------------------------Dissect Willingness---------------------------*/
      switch(tvb_get_guint8(tvb, position + 3))	{
      case 0:
	proto_tree_add_uint_format(olsr_tree, hf_olsr_willingness, tvb, position+3, 1, tvb_get_guint8(tvb, position+2), "Willingness: Never");
	break;
      case 1:
	proto_tree_add_uint_format(olsr_tree, hf_olsr_willingness, tvb, position+3, 1, tvb_get_guint8(tvb, position+2), "Willingness: Low");
	break;
      case 3:
	proto_tree_add_uint_format(olsr_tree, hf_olsr_willingness, tvb, position+3, 1, tvb_get_guint8(tvb, position+2), "Willingness: Default");
	break;
      case 6:
	proto_tree_add_uint_format(olsr_tree, hf_olsr_willingness, tvb, position+3, 1, tvb_get_guint8(tvb, position+2), "Willingness: High");
	break;
      case 7:	
	proto_tree_add_uint_format(olsr_tree, hf_olsr_willingness, tvb, position+3, 1, tvb_get_guint8(tvb, position+2), "Willingness: Always");
	break;
      default :
	proto_tree_add_uint_format(olsr_tree, hf_olsr_willingness, tvb, position+3, 1, tvb_get_guint8(tvb, position+2), "Willingness: Invalid!");
	break;
      }/* end switch Willingness */
				
      position += 4;
      message_size -= 4;
      packet_size--;
				
      while(message_size>0)   {
	int nb_neigh;
	int loop;
	int nb_neighbors;
	int before_tc;

	/*------------------------------Dissect Link Type---------------------------------- */
	switch(tvb_get_guint8(tvb, position))	{
	case 0:
	  proto_tree_add_uint_format(olsr_tree, hf_olsr_link_type, tvb, position, 1, tvb_get_guint8(tvb, position), "Link Type: Unspecified Link");
	  break;
	case 1:
	  proto_tree_add_uint_format(olsr_tree, hf_olsr_link_type, tvb, position, 1, tvb_get_guint8(tvb, position), "Link Type: Asymmetric Link");
	  break;
	case 6:
	  proto_tree_add_uint_format(olsr_tree, hf_olsr_link_type, tvb, position, 1, tvb_get_guint8(tvb, position), "Link Type: Symmetric Link");
	  break;
	case 3:
	  proto_tree_add_uint_format(olsr_tree, hf_olsr_link_type, tvb, position, 1, tvb_get_guint8(tvb, position), "Link Type: Lost Link");
	  break;
	case 10:
	  proto_tree_add_uint_format(olsr_tree, hf_olsr_link_type, tvb, position, 1, tvb_get_guint8(tvb, position), "Link Type: MPR Link");
	  break;
	case 5:
	  proto_tree_add_uint_format(olsr_tree, hf_olsr_link_type, tvb, position, 1, tvb_get_guint8(tvb, position), "Link Type: Pending");
	  break;
	default:
	  proto_tree_add_uint_format(olsr_tree, hf_olsr_link_type, tvb, position, 1, tvb_get_guint8(tvb, position), "Link Type: Invalid");
	  break;
	}/* end switch Link Type */

											
	/*----------------------Dissect Link Message Size--------------------------*/
	proto_tree_add_uint_format(olsr_tree, hf_olsr_link_message_size, tvb, position+2, 2, tvb_get_ntohs(tvb, position+2), "Link Message Size: %u bytes", tvb_get_ntohs(tvb, position+2));

	//link_message_size = (tvb_get_ntohs(tvb, offset+2) - 4) / 4;
	link_message_size = (tvb_get_ntohs(tvb, position+2) - 4) / 12;
	position += 4;
	message_size -= 4;
	packet_size--;

	nb_neigh = 0;

	//nb_neighbors = (link_message_size - 4) / (4 + 8 + 4); //IP + ADH + LQ
	nb_neighbors = link_message_size;
	before_tc = position;
	for (loop = 0; loop < nb_neighbors; loop++) {
	  guint8 lq, nlq;
	  for (i = 0; i < 8; i++)
	    orig[i] = tvb_get_guint8(tvb,i+position);
					
	  wrt_adhbytes_to_str_adholsr(orig, origin_addr_adh);
	
	  proto_tree_add_text(olsr_tree, tvb, position, 8 ,"Neighbor Adhoc Addr: %s",origin_addr_adh);
				  
	  position += 8;
	  message_size -= 8;
				  
	  tvb_memcpy(tvb, (guint8*)&neighbor_addr, position, 4);
	  proto_tree_add_ipv4(olsr_tree, hf_olsr_neighbor_addr, tvb, position, 4, neighbor_addr);

	  position +=4;
	  message_size -= 4;

	  lq  = tvb_get_guint8(tvb, before_tc + nb_neighbors * 12 + loop * 2);
	  nlq = tvb_get_guint8(tvb, before_tc + nb_neighbors * 12 + nb_neighbors * 2 + loop * 2);

	  proto_tree_add_uint(olsr_tree, hf_olsr_tclq_lq , tvb, before_tc + nb_neighbors * 12 + loop * 2    , 1, lq );
	  proto_tree_add_uint(olsr_tree, hf_olsr_tclq_nlq, tvb, before_tc + nb_neighbors * 12 + nb_neighbors * 2 + loop * 2, 1, nlq);
				  
	  packet_size--;
	}
	position += nb_neighbors * 4;
	message_size -= nb_neighbors * 4;
      }
      /* end while */
      //position = offset;
    } /* end if for Hello LQ */
    /* -------------Dissect HELLO message----------------------- */
    else if(message_size>0 && message_type == HELLO)  {
      /*---------------------Dissect Hello Emission Invertal-------------------*/
      htime = tvb_get_guint8(tvb, position+2);
      high_bits = ((htime & 0xF0) >> 4);
      low_bits = (htime & 0x0F);
      Htime = ((1<<low_bits)/16.0)*(1+(high_bits/16.0));
      proto_tree_add_double_format(olsr_tree, hf_olsr_htime, tvb, position+2, 1, Htime, "Hello Emission Interval: %.3f (in seconds)", Htime);
				
      /*-------------------------Dissect Willingness---------------------------*/
      switch(tvb_get_guint8(tvb, position+3))	{
      case 0:
	proto_tree_add_uint_format(olsr_tree, hf_olsr_willingness, tvb, position+3, 1, tvb_get_guint8(tvb, position+2), "Willingness: Never");
	break;
      case 1:
	proto_tree_add_uint_format(olsr_tree, hf_olsr_willingness, tvb, position+3, 1, tvb_get_guint8(tvb, position+2), "Willingness: Low");
	break;
      case 3:
	proto_tree_add_uint_format(olsr_tree, hf_olsr_willingness, tvb, position+3, 1, tvb_get_guint8(tvb, position+2), "Willingness: Default");
	break;
      case 6:
	proto_tree_add_uint_format(olsr_tree, hf_olsr_willingness, tvb, position+3, 1, tvb_get_guint8(tvb, position+2), "Willingness: High");
	break;
      case 7:	
	proto_tree_add_uint_format(olsr_tree, hf_olsr_willingness, tvb, position+3, 1, tvb_get_guint8(tvb, position+2), "Willingness: Always");
	break;
      default :
	proto_tree_add_uint_format(olsr_tree, hf_olsr_willingness, tvb, position+3, 1, tvb_get_guint8(tvb, position+2), "Willingness: Invalid!");
	break;
      }/* end switch Willingness */
				
      position+=4;
      message_size-=4;
      packet_size--;
				
      while(message_size>0)   {
	/*------------------------------Dissect Link Type---------------------------------- */
	switch(tvb_get_guint8(tvb, position))	{
	case 0:
	  proto_tree_add_uint_format(olsr_tree, hf_olsr_link_type, tvb, position, 1, tvb_get_guint8(tvb, position), "Link Type: Unspecified Link");
	  break;
	case 1:
	  proto_tree_add_uint_format(olsr_tree, hf_olsr_link_type, tvb, position, 1, tvb_get_guint8(tvb, position), "Link Type: Asymmetric Link");
	  break;
	case 6:
	  proto_tree_add_uint_format(olsr_tree, hf_olsr_link_type, tvb, position, 1, tvb_get_guint8(tvb, position), "Link Type: Symmetric Link");
	  break;
	case 3:
	  proto_tree_add_uint_format(olsr_tree, hf_olsr_link_type, tvb, position, 1, tvb_get_guint8(tvb, position), "Link Type: Lost Link");
	  break;
	case 10:
	  proto_tree_add_uint_format(olsr_tree, hf_olsr_link_type, tvb, position, 1, tvb_get_guint8(tvb, position), "Link Type: MPR Link");
	  break;
	case 5:
	  proto_tree_add_uint_format(olsr_tree, hf_olsr_link_type, tvb, position, 1, tvb_get_guint8(tvb, position), "Link Type: Pending");
	  break;
	default:
	  proto_tree_add_uint_format(olsr_tree, hf_olsr_link_type, tvb, position, 1, tvb_get_guint8(tvb, position), "Link Type: Invalid");
	  break;
	}/* end switch Link Type */

	/*----------------------Dissect Link Message Size--------------------------*/
	proto_tree_add_uint_format(olsr_tree, hf_olsr_link_message_size, tvb, position+2, 2, tvb_get_ntohs(tvb, position+2), "Link Message Size: %u bytes", tvb_get_ntohs(tvb, position+2));

	//link_message_size = (tvb_get_ntohs(tvb, offset+2) - 4) / 4;
	link_message_size = (tvb_get_ntohs(tvb, position+2) - 4) / 12;
	position+=4;
	message_size-=4;
	packet_size--;
											
	/*-------------------Dissect Neighbor Addresses--------------------*/
	while(link_message_size>0)   {
	  for (i = 0; i < 8; i++)
	    orig[i] = tvb_get_guint8(tvb,i+position);
					
	  wrt_adhbytes_to_str_adholsr(orig, origin_addr_adh);
	
	  proto_tree_add_text(olsr_tree, tvb, position,8 ,"  Neighbor Adhoc Addr: %s",origin_addr_adh);
	  position+=8;
	  message_size -= 8;
						
	  tvb_memcpy(tvb, (guint8*)&neighbor_addr, position, 4);
	  proto_tree_add_ipv4(olsr_tree, hf_olsr_neighbor_addr, tvb, position, 4, neighbor_addr);
	  position+=4;
	  message_size-=4;
	  packet_size--;
	  link_message_size--;

	} /* end while */
      } /* end while */
      //				position = offset;
    } /* end if for Hello */
    /*-----------------------------Dissect HNA Message--------------------------------*/
    else if(message_size>0 && message_type==HNA)	{
      while(message_size>0)	{
	tvb_memcpy(tvb, (guint8*)&network_addr, position, 4);
	proto_tree_add_ipv4(olsr_tree, hf_olsr_network_addr, tvb, position, 4, network_addr);
	message_size--;
	packet_size--;
	position+=4;
	tvb_memcpy(tvb, (guint8*)&netmask, position, 4);
	proto_tree_add_ipv4(olsr_tree, hf_olsr_netmask, tvb, position, 4, netmask);
	message_size--;
	packet_size--;
	position+=4;
      } /* end while for HNA */
      //position = offset;
    } /* end if for HNA */
			
  } /* end while for message alive */
	
  return tvb_length(tvb);
} /* end Dissecting */

void
proto_register_adholsr(void)
{                 
  static hf_register_info hf[] = {
    { &hf_olsr_packet_len,
      { "Packet Length", "olsr.packet_len",
	FT_UINT16, BASE_DEC, NULL, 0,          
	"Packet Length in Bytes", HFILL }},

    { &hf_olsr_packet_seq_num,
      { "Packet Sequence Number", "olsr.packet_seq_num",
	FT_UINT16, BASE_DEC, NULL, 0,
	"Packet Sequence Number", HFILL }},
    { &hf_olsr_origin_addr_adh,
      { "Source" ,"olsr.origin_addr_adh"
	,FT_STRING, BASE_NONE, NULL, 0x0
	,"Originator adhoc address", HFILL}},
    { &hf_olsr_message_type,
      { "Message Type", "olsr.message_type",
	FT_UINT8, BASE_DEC, VALS(message_type_vals), 0,
	"Message Type", HFILL }},

    { &hf_olsr_message_size,
      { "Message", "olsr.message_size",
	FT_UINT16, BASE_DEC, NULL, 0,          
	"Message Size in Bytes", HFILL }},

    { &hf_olsr_message_seq_num,
      { "Message Sequence Number", "olsr.message_seq_num",
	FT_UINT16, BASE_DEC, NULL, 0,          
	"Message Sequence Number", HFILL }},

    { &hf_olsr_vtime,
      { "Validity Time", "olsr.vtime",
	FT_DOUBLE, BASE_NONE, NULL, 0,
	"Validity Time", HFILL }},
		
    { &hf_olsr_ansn,
      { "Advertised Neighbor Sequence Number (ANSN)", "olsr.ansn",
	FT_UINT16, BASE_DEC, NULL, 0,
	"Advertised Neighbor Sequence Number (ANSN)", HFILL }},

    { &hf_olsr_htime,
      { "Hello emission interval", "olsr.htime",
	FT_DOUBLE, BASE_NONE, NULL, 0,
	"Hello emission interval", HFILL }},

    { &hf_olsr_willingness,
      { "Willingness to Carry and Forward", "olsr.willingness",
	FT_UINT8, BASE_DEC, NULL, 0,
	"Willingness to Carry and Forward", HFILL }},
			  
    { &hf_olsr_ttl,
      { "Time to Live", "olsr.ttl",
	FT_UINT8, BASE_DEC, NULL, 0,
	"Time to Live", HFILL }},
			  			  
    { &hf_olsr_link_type,
      { "Link Type", "olsr.link_type",
	FT_UINT8, BASE_DEC, NULL, 0,
	"Link Type", HFILL }},

    { &hf_olsr_link_message_size,
      { "Link Message Size", "olsr.link_message_size",
	FT_UINT16, BASE_DEC, NULL, 0,
	"Link Message Size", HFILL }},

    { &hf_olsr_hop_count,
      { "Hop Count", "olsr.hop_count",
	FT_UINT8, BASE_DEC, NULL, 0,
	"Hop Count", HFILL }},

    { &hf_olsr_origin_addr,
      { "Originator Address", "olsr.origin_addr",
	FT_IPv4, BASE_NONE, NULL, 0,
	"Originator Address", HFILL }},

    { &hf_olsr_neighbor_addr,
      { "  Neighbor Address", "olsr.neighbor_addr",
	FT_IPv4, BASE_NONE, NULL, 0,
	"  Neighbor Address", HFILL }},
			  
    { &hf_olsr_neighbor_addr_adh,
      { "Source" ,"olsr.neighbor_addr_adh"
	,FT_STRING, BASE_NONE, NULL, 0x0
	,"Neighbor adhoc address", HFILL}},
		
    { &hf_olsr_network_addr,
      { "Network Address", "olsr.network_addr",
	FT_IPv4, BASE_NONE, NULL, 0,
	"Network Address", HFILL }},

    { &hf_olsr_interface_addr,
      { "Interface Address", "olsr.interface_addr",
	FT_IPv4, BASE_NONE, NULL, 0,
	"Interface Address", HFILL }},

    { &hf_olsr_netmask,
      { "Netmask", "olsr.netmask",
	FT_IPv4, BASE_NONE, NULL, 0,
	"Netmask", HFILL }},

    { &hf_olsr_testlq_reserved,
      { "Test LQ reserved", "olsr.testlq_reserved",
	FT_UINT16, BASE_DEC, NULL, 0,
	"Test LQ reserved", HFILL }},

    { &hf_olsr_testlq_seqnum,
      { "Test LQ sequence number", "olsr.testlq_seqnum",
	FT_UINT16, BASE_DEC, NULL, 0,
	"Test LQ sequence number", HFILL }},

    { &hf_olsr_tclq_lq,
      { "  LQ value", "olsr.tclq_lq",
	FT_UINT16, BASE_DEC, NULL, 0,
	"  LQ value", HFILL }},

    { &hf_olsr_tclq_nlq,
      { "  Neighbor LQ value", "olsr.tclq_nlq",
	FT_UINT16, BASE_DEC, NULL, 0,
	"  Neighbor LQ value", HFILL }},
			  
  };

  static gint * ett[] = {&ett_adholsr,};

  proto_adholsr = proto_register_protocol("Ananas Adhoc Olsr Protocol","adholsr","adholsr");
	
  proto_register_field_array(proto_adholsr, hf, array_length(hf));
  proto_register_subtree_array(ett,array_length(ett));
}


void proto_reg_handoff_adholsr(void)
{
  dissector_handle_t adholsr_handle;

  adholsr_handle = create_dissector_handle(dissect_adholsr, proto_adholsr);
  dissector_add("ethertype",0x0698,adholsr_handle);
}
